
import os
import fnmatch
import logging
from odoo import api, models
from odoo.addons.saas_worker.util import version_lte
from odoo.tools.misc import file_open, get_iso_codes
from odoo.tools.translate import TranslationModuleReader
from odoo.tools.translate import TranslationFileReader

try:
    from odoo.tools.translate import WEB_TRANSLATION_COMMENT
except ImportError:  # > 17.0
    WEB_TRANSLATION_COMMENT = 'openerp-web'
try:
    from odoo.tools.translate import JAVASCRIPT_TRANSLATION_COMMENT
except ImportError:  # < 16.0
    JAVASCRIPT_TRANSLATION_COMMENT = 'odoo-javascript'


_logger = logging.getLogger(__name__)

if version_lte("saas~15.4"):  # >= 14.0, < 16.0

    # Stripped down version of "CodeTranslations" of version 16.0 to only
    # read web translations
    class CodeTranslations:
        def __init__(self):
            # {(module_name, lang): {'message': [{'id': src, 'string': value}]}
            self.web_translations = {}

        @staticmethod
        def _read_code_translations_file(fileobj, filter_func):
            """ read and return code translations from fileobj with filter filter_func

            :param func filter_func: a filter function to drop unnecessary code translations
            """
            # current, we assume the fileobj is from the source code, which only contains the translation for the current module
            # don't use it in the import logic
            translations = {}
            fileobj.seek(0)
            reader = TranslationFileReader(fileobj, fileformat='po')
            for row in reader:
                if row.get('type') == 'code' and row.get('src') and filter_func(row):
                    translations[row['src']] = row['value']
            return translations

        @staticmethod
        def _get_code_translations(module_name, lang, filter_func):
            lang_base = lang.split('_')[0]
            po_paths = [f"{module_name}/i18n/{lang_base}.po",
                        f"{module_name}/i18n/{lang}.po"]
            translations = {}
            for po_path in po_paths:
                try:
                    with file_open(po_path, mode='rb') as fileobj:
                        try:
                            p = CodeTranslations._read_code_translations_file(fileobj, filter_func)
                            translations.update(p)
                        except IOError:
                            iso_lang = get_iso_codes(lang)
                            filename = '[lang: %s][format: %s]' % (iso_lang or 'new', 'po')
                            _logger.exception("couldn't read translation file %s", filename)
                except (IOError, FileNotFoundError):
                    continue
            return translations

        def _load_web_translations(self, module_name, lang):
            def filter_func(row):
                return row.get('value') and (
                    JAVASCRIPT_TRANSLATION_COMMENT in row['comments']
                    or WEB_TRANSLATION_COMMENT in row['comments']
                )
            translations = CodeTranslations._get_code_translations(module_name, lang, filter_func)
            self.web_translations[(module_name, lang)] = {
                "messages": [{"id": src, "string": value} for src, value in translations.items()]
            }

        def get_web_translations(self, module_name, lang):
            if (module_name, lang) not in self.web_translations:
                self._load_web_translations(module_name, lang)
            return self.web_translations[(module_name, lang)]

    code_translations = CodeTranslations()

    class IrTranslation(models.Model):
        _inherit = 'ir.translation'

        @api.model
        def get_translations_for_webclient(self, mods, lang):
            if not lang:
                lang = self._context.get("lang")
            if not lang:
                all_langs = self.env['res.lang'].get_installed()
                lang = all_langs[0][0] if all_langs else 'en_US'
            translations_per_module, lang_params = super().get_translations_for_webclient(mods, lang)
            for module in (mod for mod in mods if mod.startswith('saas_')):
                # as we can't update module on saas to load new translations
                # into ir.translation, do like in 16.0 and load it directly
                # from the PO file for saas modules
                translations_per_module[module] = code_translations.get_web_translations(module, lang)

            return translations_per_module, lang_params

_orig_export_translatable_resources = TranslationModuleReader._export_translatable_resources

def _export_translatable_resources(self):
    _orig_export_translatable_resources(self)

    # Also extract transaltions from non-standard paths: saas_*/static/{xml,js}
    # Note: _path_list is setup by the parent (orignal) method
    for (path, recursive) in self._path_list:
        for root, _dummy, files in os.walk(path, followlinks=True):
            if fnmatch.fnmatch(root, '*/saas_*/static/js*'):
                # Javascript source files
                for fname in fnmatch.filter(files, '*.js'):
                    self._babel_extract_terms(
                        fname, path, root, 'javascript',
                        extra_comments=[WEB_TRANSLATION_COMMENT, JAVASCRIPT_TRANSLATION_COMMENT],
                        extract_keywords={'_t': None, '_lt': None}
                    )
            if fnmatch.fnmatch(root, '*/saas_*/static/xml*'):
                # QWeb template files
                for fname in fnmatch.filter(files, '*.xml'):
                    self._babel_extract_terms(
                        fname, path, root, 'odoo.tools.translate:babel_extract_qweb',
                        extra_comments=[WEB_TRANSLATION_COMMENT, JAVASCRIPT_TRANSLATION_COMMENT]
                    )
            if not recursive:
                # due to topdown, first iteration is in first level
                break


TranslationModuleReader._export_translatable_resources = _export_translatable_resources
